/*  SirvNET Services' main header
 *
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */

#ifndef SERVSTRUCT_H
#define SERVSTRUCT_H


#include "config.h"

/*****************************************************************************/

typedef struct chaninfo_ ChanInfo;
typedef struct channel_ Channel;
typedef struct nickinfo NickInfo;
typedef struct user_ User;
typedef struct server_list ServerList;
typedef struct email_list EmailList;
typedef struct usernode UserNode;
typedef struct channelnode ChannelNode;

struct user_ {
        User           *next, *prev;
	UserNode       *node;
        char            nick[NICKMAX];
#ifdef SOLID
	char           *maskhost;
#endif
        char           *username;
        char           *host;
        char           *realname;
        char           *server;
	long            timestamp;
	long            mode;
 #ifdef BAHAMUT
	unsigned long   ip;
 #else
	char           *ip;
 #endif
	ServerList     *slptr;
	int             nchanges;
        time_t          signon;
        time_t          flood_time;
        short           floodlev, floodlev2;
        short           motd, version;
	short           invalid_nicks;
	short           invalid_chans;
	time_t          invalid_time;
	time_t          lastreg;
	time_t          lastmemo;
	unsigned int    memos;
	char           *os_nick;
	unsigned int    os_level;
	unsigned int    news;
	struct u_idlist {
		struct u_idlist *next, *prev;
		char *nick;
	} *nicks;
	struct u_founderlist {
		struct u_founderlist *next, *prev;
		ChanInfo *chan;
	} *fchans;
	struct UserChans {
		struct UserChans *next, *prev;
		Channel *chan;
	} *chans;
};


#define UMODE_O              0x00000001
#define UMODE_R              0x00000002
#define UMODE_a              0x00000004
#define U_MEMO_NOTIFIED      0x40000000
#define U_RECOGNIZED         0x80000000



/*****************************************************************************/

struct nickinfo {
	NickInfo *forward;
	char *nick;
	long flags;
	unsigned long id;
	int remind;
	unsigned int news;
	struct NickAccess {
		struct NickAccess *next, *prev;
		char *mask;
	} *entries;
};


/*****************************************************************************/

struct usernode {
        UserNode *left, *right, *back;
        User *u;
        unsigned long value;
};

/*****************************************************************************/

struct channelnode {
        ChannelNode *left, *right, *back;
        Channel *c;
        unsigned long value;
};

/*****************************************************************************/

struct channel_ {
	Channel *next, *prev;
	ChannelNode *node;
	char name[CHANMAX];
	long modes;
	char *chankey;
	int chanlimit;
	int activebans;
	time_t stamp;
	struct ChanUsers {
		struct ChanUsers *next, *prev;
		User *user;
 #ifndef HAS_HALFOP
	} *users, *ops, *voices;
 #else
	} *users, *ops, *voices, *halfs;
 #endif
	struct ChanBans {
		struct ChanBans *next, *prev;
		char *mask;
		time_t btime;
	} *bans;

};

#define CMODE_r 0x00000001
#define CMODE_R 0x00000002
#define CMODE_T 0x00000004
#define CMODE_N 0x00000008
#define CMODE_I 0x00000010
#define CMODE_K 0x00000020
#define CMODE_S 0x00000040
#define CMODE_P 0x00000080
#define CMODE_M 0x00000100
#define CMODE_L 0x00000200
#define CMODE_J 0x00000400
#define CMODE_C 0x00000800
#define CMODE_o 0x00001000
#define CMODE_m 0x00002000

/*****************************************************************************/

struct chaninfo_ {
	char name[CHANMAX];
	char founder[NICKMAX];
	unsigned long fid;   /* founder's nick id */
	long flags;
	long id;
	int mlevel;
	int remind;
	int topiclock;
	char *topic;
	char topic_setter[NICKMAX];
	time_t topic_time;
	char *welcome;
	struct AccessList {
		struct AccessList *next, *prev;
		char *entry;
		char *instater;
		int level;
		unsigned long nickid;
		long when;
		char *reason;
	} *entries;
};


/*****************************************************************************/

typedef struct bademails BadEmails;
struct bademails {
        char *host;
        struct bademails *next;
};

/*****************************************************************************/

typedef struct masters Masters;
struct masters {
        char *nick;
        struct masters *next;
};

/*****************************************************************************/

typedef struct closures CloseChans;
struct closures {
	struct closures *next, *prev;
	char *chan;
	time_t expiry;
};

/*****************************************************************************/

typedef struct nfreeze nfreezes;
struct nfreeze {
	char *ent;
	struct nfreeze *next, *prev;
};

/*****************************************************************************/

typedef struct akills AKill;
struct akills {
	char *mask;
	long flag;
	unsigned long id;
	struct akills *next, *prev;
};


#define AK_WILD_IDENT 0x00000001
#define AK_WILD_HOST  0x00000002


/*****************************************************************************/

typedef struct ignores Ignore;
struct ignores {
	char *mask;
	struct ignores *next, *prev;
};

/*****************************************************************************/

struct server_list {
	char *name;
	ServerList *next, *prev, *hub, **downlinks;
	int ndownlinks, downlink_size;
	User **users;
	int nusers, users_size;
};


/*****************************************************************************/

struct email_list {
	EmailList *next;
	char *info, *subject, *email;
	int append;
	int sock;
	int retries;
};


/*****************************************************************************/

#endif  /* SERVSTRUCT_H */

