/*
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */


#include "services.h"
#include "log.h"

/******************************************************************************************/


typedef struct trigger Trigger;
struct trigger {
	char *mask;
	int value;
	struct trigger *prev, *next;
};

Trigger *trigger_list = NULL;

struct clone {
	char *host;
	long time;
#ifdef BAHAMUT
	unsigned long ip;
#endif
};

static struct clone warnings[DETECT_CLONE_SIZE];

static void triggerm(const char *mask, int value);

/******************************************************************************************/

void load_triggers()
{
	int num_rows;
	Trigger *trigger;

	if ((num_rows=sql_select("mask,number FROM triggers"))) {
		int i;
		for (i=0; i < num_rows; i++) {
			row = fetch(res);
			trigger = scalloc(sizeof(Trigger),1);
			trigger->mask = sstrdup(row[0]);
			trigger->value = atoi(row[1]);
			trigger->next = trigger_list;
			if (trigger_list)
				trigger_list->prev = trigger;
			trigger_list = trigger;
		}
	}
}

/******************************************************************************************/

static int ckill_time(const char *clonetime)
{
	int amount = strtol(clonetime, (char **)&clonetime, 10);

	switch(*clonetime) {
		case 'd': amount *= 24;
		case 'h': amount *= 60;
		case 'm': amount *= 60; break;
		default : amount = 1800;
	}
	return amount;
}

/******************************************************************************************/


#ifdef BAHAMUT

typedef struct username_ UserName;
struct username_ {
	struct username_ *next;
	char *username;
	int names;
};

typedef struct userhost UserHost;
struct userhost {
	struct userhost *left, *right, *back;
	struct username_ *usernames;
	unsigned long ip;
	int hosts;
};

UserHost *userhosts = NULL;


int username_list(User *user, UserHost *uh, int add)
{
	UserName *un = uh->usernames, *last=NULL;

	while (un != NULL) {
		if (sstrcmp(un->username, user->username) == 0) {
			if (add) {
				un->names++;
				return un->names;
			}
			un->names--;
			if (!un->names) {
				if (last)
					last->next = un->next;
				else
					uh->usernames = un->next;
				free(un->username); free(un);
				return 0;
			}
			return un->names;
		}
		last = un;
		un = un->next;
	}
	if (!add)
		return 0;

	un = scalloc(sizeof(UserName), 1);
	un->username = sstrdup(user->username);
	un->names = 1;
	un->next = uh->usernames;
	uh->usernames = un;
	return 1;
}


void userhost_list(User *user, int add, int *hosts, int *names)
{
	UserHost *uh = userhosts, *newnode;


	if (!uh) {
		uh = scalloc(sizeof(UserHost), 1);
		uh->ip = 0;
		userhosts = uh;
	}

	while (uh != NULL) {
		if (uh->ip == user->ip) {
			if (add) {
				uh->hosts++;
				*names = username_list(user, uh, ADD);
				*hosts = uh->hosts;
				return;
			}
			uh->hosts--;
			username_list(user, uh, DEL);
			if (!uh->hosts) {
				UserName *un = uh->usernames, *next;
				if (uh->back->right == uh)
					uh->back->right = uh->right;
				else
					uh->back->left = uh->right;
				if (uh->right)
					uh->right->back = uh->back;
				if (uh->left) {
					UserHost *nptr = uh->back, *movenode = uh->left;

					while (nptr != NULL) {
						if (movenode->ip < nptr->ip) {
							if (nptr->left)
								nptr = nptr->left;
							else {
								nptr->left = movenode;
								movenode->back = nptr;
								return;
							}
						} else if (movenode->ip >= nptr->ip) {
							if (nptr->right)
								nptr = nptr->right;
							else {
								nptr->right = movenode;
								movenode->back = nptr;
								return;
							}
						}
					}
				}
				while (un) {
					next = un->next;
					free(un->username);
					free(un);
					un = next;
				}
				free(uh);
				return;
			}
			return;
		}
		if (uh->ip < user->ip) {
			if (uh->left)
				uh = uh->left;
			else
				break;
		} else if (uh->ip >= user->ip) {
			if (uh->right)
				uh = uh->right;
			else
				break;
		}
	}
	if (!add || !uh)
		return;

	newnode = scalloc(sizeof(UserHost), 1);
	newnode->ip = user->ip;
	newnode->hosts = 1;
	newnode->back = uh;
	username_list(user, newnode, ADD);

	if (uh->ip < user->ip)
		uh->left = newnode;
	else
		uh->right = newnode;

	*hosts = 1;
	*names = 1;

	return;
}

/******************************************************************************************/


void check_clones(User *user) {
	Trigger *trigger;
	char idbuf[256], *ipbuf;
	int i, ipcount=0, identcount=0;
	unsigned long x = htonl(user->ip);
	int ipalarm = CLONE_MIN_USERS;
	int identalarm = CLONE_MIN_USERS;

	if (!x)  /* oper host mask 0.0.0.0 */
		return;

	userhost_list(user, ADD, &ipcount, &identcount);


	snprintf(idbuf, sizeof(idbuf), "%s@%ld.%ld.%ld.%ld", user->username,
		(long)(x>>0) & 0xff, (long)(x>>8) & 0xff,
		(long)(x>>16) & 0xff, (long) (x>>24) & 0xff);

	ipbuf = strchr(idbuf, '@'); *ipbuf++;



/* Check if alarm is > trigger value below incase there are multiple trigger enteries that match the buffers.
   If so, always go with the lowest trigger. */

	for (trigger = trigger_list; trigger; trigger = trigger->next) {
		if (match_wild_nocase(trigger->mask, ipbuf)) {
			if (ipalarm > trigger->value || ipalarm == CLONE_MIN_USERS)
				ipalarm = trigger->value;
		}
		if (match_wild_nocase(trigger->mask, idbuf)) {
			if (identalarm > trigger->value || identalarm == CLONE_MIN_USERS)
				identalarm = trigger->value;
		}
	}

/* Check to see if ipalarm and identalarm are equal to CLONE_MIN_USERS to determine if the host is
   triggered or not. If triggered, don't akill - just warn. */
	if (CKILL_COUNT && (ipalarm == CLONE_MIN_USERS) && (identalarm == CLONE_MIN_USERS)
		&& ((identcount >= CKILL_COUNT) || (ipcount >= CKILL_COUNT))) {
		char *bufptr;

		if (ipcount >= CKILL_COUNT)
			bufptr = ipbuf;
		else
			bufptr = idbuf;



		if (!(sql_select("mask FROM akill WHERE mask = '%s%s'", 
			ipcount>=CKILL_COUNT ? "*@" : "", bufptr))) {
			char *reason = "Clones detected from your host.";
			char *code = create_ticket();
			unsigned long id=0;

			send_sql("INSERT INTO akill VALUES (%d,'%s%s','%d','%d','%s','%s','%s')",
				(id=generate_primary_id("akill")),ipcount>=CKILL_COUNT ? "*@" : "", bufptr, 
				time(NULL),(ckill_time(CKILL_TIME)+time(NULL)),CKILL_REASON, CKILL_WHO, code);
			akills(bufptr,1,id);
			globops(NULL, "Added timed [%s] automatic clone AKILL for '%s%s'",
				CKILL_TIME, ipcount>=ipalarm ? "*@" : "", bufptr);
			ircd_akill(CKILL_WHO, bufptr, reason, code, ckill_time(CKILL_TIME));
			free(code);
			return;
		}

	}
/* if ipcount == identcount then no point in warning about ident - we'll warn about the ip further below  */
	if ((identcount >= identalarm) && (ipcount != identcount)) {
		for (i = DETECT_CLONE_SIZE-1; i >= 0; --i) {
			if (warnings[i].host && !sstrcmp(warnings[i].host, idbuf)) {
				if (warnings[i].time > (time(NULL) - 10))
					return;
			}
		}
		i = DETECT_CLONE_SIZE-1;
		if (warnings[0].host)
			free(warnings[0].host);
		bcopy(warnings+1, warnings, sizeof(struct clone) * i);
		
		warnings[i].host = sstrdup(idbuf);
		warnings[i].time = time(NULL);
		warnings[i].ip = user->ip;
		
		globops(NULL, D_IPCLONES,
			idbuf, identcount, s_OperServ, idbuf);
		if (debug)
			log("Possible clones detected from %s [%d]",
				idbuf, identcount);

	}
	if (ipcount >= ipalarm) {
		for (i = DETECT_CLONE_SIZE-1; i >= 0; --i) {
			if (warnings[i].host && !sstrcmp(warnings[i].host, ipbuf)) {
				if (warnings[i].time > (time(NULL) - 10))
					return;
			}
		}
		i = DETECT_CLONE_SIZE-1;
		if (warnings[0].host)
			free(warnings[0].host);
		bcopy(warnings+1, warnings, sizeof(struct clone) * i);
		
		warnings[i].host = sstrdup(ipbuf);
		warnings[i].time = time(NULL);
		warnings[i].ip = user->ip;
		
		globops(NULL, D_IPCLONES,
			ipbuf, ipcount, s_OperServ, ipbuf);
		if (debug)
			log("Possible clones detected from %s [%d]",
				ipbuf, ipcount);

	}
}


#else  /* !BAHAMUT */

void check_clones(User *user)
{
	User *u;
	Trigger *trigger;
	char buf[512], hbuf[512];
	int i, host_count=0, ident_count=0, triggered=0, ulist=0;
	int halarm = CLONE_MIN_USERS;
	int ialarm = CLONE_MIN_USERS;

	if (!DETECT_CLONES)
		return;

	snprintf(buf, sizeof(buf), "%s@%s", user->username, user->host);
	snprintf(hbuf, sizeof(hbuf), "*@%s", user->host);

	for (ulist=0; ulist < 256; ulist++) {
		for (u = userlist[ulist]; u; u = u->next) {
			if (match_wild_nocase(u->host, user->host)) {
				host_count++;
				if (match_wild_nocase(u->username, user->username))
					ident_count++;
			}
		}
	}


	for (trigger = trigger_list; trigger; trigger = trigger->next) {
		if (match_wild_nocase(trigger->mask, buf)) {
			triggered=1;
			if (host_count > trigger->value || halarm == CLONE_MIN_USERS)
				halarm = trigger->value;
			if (ident_count > trigger->value || ialarm == CLONE_MIN_USERS)
				ialarm = trigger->value;

		}
	}
	if (CKILL_COUNT && !triggered && ((host_count >= CKILL_COUNT) || (ident_count >= CKILL_COUNT))) {
		char *bufptr;

		if (host_count >= CKILL_COUNT)
			bufptr = hbuf;
		else
			bufptr = buf;


		if (!(sql_select("mask FROM akill WHERE mask = '%s'", bufptr))) {
			unsigned long id=generate_primary_id("akill");
			char *code = create_ticket();
			akills(bufptr,1,id);
			send_sql("INSERT INTO akill VALUES (%d,'%s','%d','%d','%s','%s','%s')",
				id, bufptr, time(NULL),
				(ckill_time(CKILL_TIME)+time(NULL)),CKILL_REASON, CKILL_WHO, code);
			globops(NULL, "Added timed [%s] automatic clone AKILL for '%s'",
				CKILL_TIME, bufptr);
			free(code);
			mass_check_akill();
			return;
		}

	}


/* if host_count == ident_count then no point in warning about ident - we'll warn about the ip further below  */
	if ((ident_count >= ialarm) && (host_count != ident_count)) {
		for (i = DETECT_CLONE_SIZE-1; i >= 0; --i) {
			if (warnings[i].host && !sstrcmp(warnings[i].host, buf)) {
				if (warnings[i].time > (time(NULL) - 10))
					return;
			}
		}
		i = DETECT_CLONE_SIZE-1;
		if (warnings[0].host)
			free(warnings[0].host);
		bcopy(warnings+1, warnings, sizeof(struct clone) * i);
		
		warnings[i].host = sstrdup(buf);
		warnings[i].time = time(NULL);
		
		globops(NULL, D_CLONES, buf, ident_count);
		if (debug)
			log("Possible clones detected from %s [%d]",
				buf, ident_count);

	}
	if (host_count >= halarm) {
		for (i = DETECT_CLONE_SIZE-1; i >= 0; --i) {
			if (warnings[i].host && !sstrcmp(warnings[i].host, hbuf)) {
				if (warnings[i].time > (time(NULL) - 10))
					return;
			}
		}
		i = DETECT_CLONE_SIZE-1;
		if (warnings[0].host)
			free(warnings[0].host);
		bcopy(warnings+1, warnings, sizeof(struct clone) * i);
		
		warnings[i].host = sstrdup(hbuf);
		warnings[i].time = time(NULL);
		
		globops(NULL, D_CLONES,
			hbuf, host_count);
		if (debug)
			log("Possible clones detected from %s [%d]",
				hbuf, host_count);

	}
}

#endif /* #ifdef BAHAMUT */


/******************************************************************************************/

void do_trigger(const char *source, const char *service)
{
	const char *mask = gettok(NULL, " ", 100);
	const char *value = strtok(NULL, " ");
	const char *reason = strtok(NULL, "");
	int cvalue=0;

	if (mask && value && sstrcmp(mask, "LIST"))
		cvalue = strtol(value, (char **)&value, 10);


	if (!mask || (sstrcmp(mask, "LIST") && !value)) {
		notice(service, source,
			"Syntax: TRIGGER LIST <pattern>  or  TRIGGER <user@host> <value> [<reason>]");
		notice(service, source,
			"\2/%s OHELP TRIGGER\2 for more information.",
			service);

	} else if (sstrcmp(mask, "LIST") == 0) {
		char *ptr, *cptr;
		int num_rows;
		notice(service, source,
			"\2TRIGGER listing\2: (Search pattern: %s)", value ? value : "*");

		if (!value)
			value = "%";

		cptr = sstrdup(value);
		ptr = cptr;

		while (*ptr) {
			if (*ptr == '*')
				*ptr = '%';
			*ptr++;
		}
		if (!(num_rows=sql_select("* FROM triggers WHERE mask LIKE '%s' ORDER BY mask", cptr)))
			notice(service, source,
				"No triggers matching search pattern.");

		else {
			int i;
			for (i=0; i < num_rows; i++) {
				row = fetch(res);
				notice(service, source,
					"Mask:  %s   Value: %s  Placed by: %s",
					row[0], row[1], row[3]);
				notice(service, source,
					"   Reason: %s", row[2]);
			}
			notice(service, source, "\2End of TRIGGER list.");
		}
		free(cptr);

	} else if (cvalue < 0 || cvalue > 500) {
		notice(service, source,
			"TRIGGER ERROR: Value of trigger out of range. (0-500 valid)");

	} else if (reason && strlen(reason) > 255) {
		notice(service, source,
			"TRIGGER ERROR: Reason for trigger too long.");

	} else if (!check_valid_mask(mask) && !check_valid_host(mask) && !check_valid_ip(mask)) {
		notice(service, source,
			"TRIGGER ERROR: Invalid host@mask or ip.");

	} else {
		int num_rows = sql_select("number FROM triggers WHERE mask = '%s'", mask);

		if (num_rows) {
			row = fetch(res);
			if (cvalue == atoi(row[0])) {
				notice(service, source,
					"Trigger value already set to '%d' for mask '%s'",
					cvalue, mask);
				return;
			} else if (cvalue == 0 || cvalue == CLONE_MIN_USERS) {
				globops(NULL, "%s has reset trigger for '%s'",
					opertrace(finduser(source),0), mask);
				notice(service, source,
					"Trigger for '%s' has been reset.", mask);
				send_sql("DELETE FROM triggers WHERE mask = '%s'", mask);
				triggerm(mask,0);

			} else {
				globops(NULL, "%s has re-triggered '%s' %s->%d", 
					opertrace(finduser(source),0), mask, row[0], cvalue);

				if (!reason)
					send_sql("UPDATE triggers SET number = %d WHERE mask = '%s'",
						cvalue, mask);
				else
					send_sql("UPDATE triggers SET number = %d, reason = '%s' "
						"WHERE mask = '%s'",
						cvalue, reason, mask);
				triggerm(mask,cvalue);
			}
		} else {
			char *ptr = strchr(mask, '@');

			if (ptr)
				*ptr++;

			if (cvalue == 0)
				notice(service, source,
					"TRIGGER ERROR: Mask '%s' not found on trigger list.", mask);

			else if (cvalue == CLONE_MIN_USERS)
				notice(service, source,
					"TRIGGER ERROR: Value of '%d' is the default trigger for all hosts.",
						CLONE_MIN_USERS);

			else if ((strspn(mask, "1234567890@abcdefghijklmnopqrstuvwxyz"
				"ABCDEFGHIJKLMNOPQRSTUVWXYZ-*.") != strlen(mask)))
				notice(service, source,
					"TRIGGER ERROR: Invalid mask - Illegal characters.");

			 else if (ptr && (strchr(ptr, '@') || *ptr == '-'))
				notice(service, source,
					"TRIGGER ERROR: Invalid mask.");

			else {
				char buf[256];

				snprintf(buf, sizeof(buf), "@%s", ptr ? ptr : mask);
				if (!check_valid_mask(buf)) {
					notice(service, source,
						"TRIGGER ERROR: Invalid mask.");
					return;
				}
				send_sql("INSERT INTO triggers VALUES ('%s',%d,'%s','%s')",
					mask, cvalue, reason ? reason : "Not specified.",
					opertrace(finduser(source),1));
				globops(NULL, "'%s' has triggered '%s' %d->%d",
					opertrace(finduser(source),0), mask, CLONE_MIN_USERS, cvalue);
				notice(service, source, "Triggered '%s' %d->%d.",
					mask, CLONE_MIN_USERS, cvalue);
				triggerm(mask,cvalue);
			}

		}
	}
}

/******************************************************************************************/

static void triggerm(const char *mask, int value)
{
	Trigger *trigger;

	if (!mask)
		return;

	for (trigger = trigger_list; trigger; trigger = trigger->next) {
		if (sstrcmp(trigger->mask, mask) == 0) {
			if (value) {
				trigger->value = value;
				return;
			} else {
				if (trigger->prev)
					trigger->prev->next = trigger->next;
				else
					trigger_list = trigger->next;
				if (trigger->next)
					trigger->next->prev = trigger->prev;
				free(trigger->mask); free(trigger);
				return;
			}
		}
	}
	if (!value) {
		if (debug)
			log("Mask '%s' not found on internal trigger list!", mask);
		return;
	}
	trigger = scalloc(sizeof(Trigger),1);
	trigger->mask = sstrdup(mask);
	trigger->value = value;
	trigger->next = trigger_list;
	if (trigger_list)
		trigger_list->prev = trigger;
	trigger_list = trigger;

}
/******************************************************************************************/

#ifdef BAHAMUT
void do_clones(const char *source, const char *service)
{

	const char *mask = gettok(NULL, " ", 100);
	char *ident=0, *s=0;
	int i, shown=0, found=0, ulist=0;
	User *user;

	if (!mask) {
		notice(service, source,
			"Syntax: CLONES [ident@]<ip>");
		notice(service, source,
			"\2/%s OHELP CLONES\2 for more information.",
			service);
		return;
	}
	if (strchr(mask, '@')) {
		s = sstrdup(mask);
		ident = strtok(s, "@");
	}

	for (i = DETECT_CLONE_SIZE-1; i >= 0; --i) {
		if (warnings[i].host && !sstrcmp(warnings[i].host, mask)) {
			found=1;
			notice(service, source,
				"\2CLONES information on: %s\2", warnings[i].host);
			notice(service, source,
				"Time last seen clones: %s", timebuf(warnings[i].time));
			for (ulist=0; ulist < 256; ulist++) {
				for (user = userlist[ulist]; user; user = user->next) {
					if (s) {
						if ((user->ip == warnings[i].ip) && !sstrcmp(ident, user->username)) {
							notice(service, source,
								"  %s		(%s@%s)",
								user->nick, user->username, user->host);
							shown=1;
						}
					} else if (user->ip == warnings[i].ip) {
							notice(service, source,
								"  %s		(%s@%s)",
								user->nick, user->username, user->host);
							shown=1;
					}
				}
			}
		break;
		}
	}
	if (!found)
		notice(service, source, "Clone information not found for '%s' "
					"(Too far back in history or invalid ident@ip)", mask);
	else if (!shown)
		notice(service, source, "Clones from '%s' no longer online.", mask);
	else if (found)
		notice(service, source, "\2End of CLONES information.");
	if (s)
		free(s);
}

#endif


/******************************************************************************************/
/******************************************************************************************/

