/*
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */


#include "services.h"
#include "log.h"
#include "timeout.h"


EmailList *email_list = NULL;


/************************************************************************/
/************************************************************************/

static void smtp_send(const char *info, const char *subject, const char *email, int append, int smtp_retry)
{
	struct sockaddr_in smtp_addr;
	struct hostent *smtph = gethostbyname(SMTP_SERVER);
	int smtp;
	long flags=0;
	EmailList *einfo;

	bzero(&smtp_addr, sizeof(smtp_addr));
	bcopy(smtph->h_addr, (char *)&smtp_addr.sin_addr, smtph->h_length);
	smtp_addr.sin_family = smtph->h_addrtype;
	smtp_addr.sin_port = htons(25);
	smtp_addr.sin_addr.s_addr = inet_addr(SMTP_SERVER);

	smtp = socket(AF_INET, SOCK_STREAM, 0);

	fcntl(smtp, F_GETFL, &flags);
	flags |= O_NONBLOCK;
	fcntl(smtp, F_SETFL, flags);

	FD_SET(smtp, &smtp_fdlist);
	if (smtp > smtp_fdmax)
		smtp_fdmax = smtp;


	connect(smtp, (struct sockaddr *)&smtp_addr, sizeof(smtp_addr));

	einfo = scalloc(sizeof(EmailList), 1);
	einfo->info = sstrdup(info);
	einfo->subject = sstrdup(subject);
	einfo->email = sstrdup(email);
	einfo->append = append;
	einfo->sock = smtp;
	einfo->retries = smtp_retry;
	einfo->next = email_list;
	email_list = einfo;

}


static void pipe_send(const char *info, const char *subject, const char *email, int append)
{
	char time_buf[64];
	char buffer[1024];
	time_t now = time(NULL);
	struct tm tm = *localtime(&now);
	FILE *sig = fopen(SERVICES_SIG, "r");
	FILE *mailpipe = popen(SENDMAIL_PATH, "w");


	strftime(time_buf, sizeof(time_buf), "%a %b %d %H:%M:%S %Y %Z", &tm);
	time_buf[sizeof(time_buf)-1] = 0;


	fprintf(mailpipe, "Date: %s\n", time_buf);
	fprintf(mailpipe, "From: %s <%s>\n", NETWORK_NAME, RETURN_EMAIL);
	fprintf(mailpipe, "To: %s\n", email);
	fprintf(mailpipe, "Subject: %s\n\n", subject);


	if (append==0 && HIGH_NICK_SECURITY) {
		FILE *high = fopen(QUESTION_FILE, "r");
		if (high) {
			while (!feof(high)) {
				fgets(buffer, 1024, high);
				fprintf(mailpipe, buffer);
			}
			fclose(high);
		}
	}

	fprintf(mailpipe, "%s\n\n", info);

	if (append < 2) {
		FILE *custom = fopen(CUSTOM_AUTH, "r");
		if (custom) {
			while (!feof(custom)) {
				fgets(buffer, 1024, custom);
				fprintf(mailpipe, buffer);
			}
			fclose(custom);
		}
	}

	if (sig) {
		while (!feof(sig)) {
			fgets(buffer, 1024, sig);
			fprintf(mailpipe, buffer);
		}
		fclose(sig);
	}


	pclose(mailpipe);

}

/************************************************************************/

inline void e_mail(const char *info, const char *subject, const char *email, int append)
{
	if (SMTP_SERVER)
		smtp_send(info, subject, email, append, 0);
	else
		pipe_send(info, subject, email, append);
}

/************************************************************************/

/*
 *  E-Mail control - Only allow x e-mails per day for specific routines
 *
 *  Type 1 - Authorization
 *  Type 2 - General e-mails
 *  Type 3 - Sendpass attempts
 *
 */

static unsigned int nemails = 0;
static unsigned int email_size = 0;

static struct email_control {
	char *info;
	int type;
	int times;
	time_t when;
} *emails = NULL;



void add_email(const char *info,int type)
{
	if (nemails >= email_size) {
		if (email_size < 8)
			email_size = 8;
		else
			email_size *= 2;
		emails = srealloc(emails, sizeof(*emails) * email_size);
	}

	emails[nemails].info = sstrdup(info);
	emails[nemails].type = type;
	emails[nemails].times = 1;
	emails[nemails].when = time(NULL);
	nemails++;
}

int check_email(const char *info, int type, int incr)
{
	unsigned int i;
	for (i=0; i < nemails; i++) {
		if (!sstrcmp(info, emails[i].info) && (emails[i].type == type)) {
			if (incr)
				emails[i].times += 1;
			return emails[i].times;
		}
	}
	return 0;
}


/* Remove tracked emails that have been around for awhile */
void kill_emails()
{
	unsigned int i;


	for (i=0; i < nemails; i++) {
		if ((time(NULL) - emails[i].when) > BLOCK_EMAIL_TIME) {
			free(emails[i].info);
			nemails--;
			if (i < nemails)
				bcopy(emails+i+1, emails+i, sizeof(*emails) * (nemails-i));
			i--;
		}
	}

}

/************************************************************************/

/* Try our best to determine if the email address is valid */

int check_valid_email(const char *email)
{
	char *ptr = strchr(email, '@'), *ptr2;

	if (ptr)
		ptr2 = strchr(ptr, '.');


	if (!strchr(email, '@') || !strchr(email, '.') || strchr(++ptr, '@'))
		return 0;

	else if (!ALLOW_LITERAL && (strspn(ptr, "1234567890.") == strlen(ptr)))
		return 0;

	else if (ALLOW_LITERAL && (strspn(ptr, "1234567890.") == strlen(ptr)) && !check_valid_ip(ptr))
		return 0;

	else if ((strspn(email, "1234567890abcdefghijklmnopqrstuvwxyz"
			"ABCDEFGHIJKLMNOPQRSTUVWXYZ_.-@") != strlen(email)))
		return 0;
	else if (strspn(*&email, "1234567890@") > 0)
		return 0;
	else if ((strlen(email) < 6) || (strlen(email) > EMAILMAX))
		return 0;
	else
		return 1;

}

/************************************************************************/

int check_bad_email(const char *email)
{
	BadEmails *bad;

	for (bad = bademails; bad; bad = bad->next) {
		if (bad->host && (match_wild_nocase(bad->host, email)))
			return 1;
	}
	return 0;
}

/************************************************************************/

void remove_emailinfo(EmailList *einfo, EmailList *last)
{
	if (!einfo)
		return;

	if ((einfo==email_list) || (last && (last==einfo)))
		last=NULL;

	else if (!last)
		for (last=email_list; last && einfo != last->next; last = last->next);


	if (einfo->sock != -1) {  /* Already shutdown - this is a (retry) structure delete only */
		FD_CLR(einfo->sock, &smtp_read_fds);
		shutdown(einfo->sock, SHUT_RDWR);
		close(einfo->sock);
	}

	if (last)
		last->next = einfo->next;
	else if (email_list == einfo)
			email_list = einfo->next;


	free(einfo->info);free(einfo->subject);free(einfo->email);free(einfo);

}

/************************************************************************/

static void resend_smtp(Timeout *t)
{
	EmailList *einfo = t->data;

	smtp_send(einfo->info, einfo->subject, einfo->email, einfo->append, einfo->retries);
	remove_emailinfo(einfo, NULL);

}


/************************************************************************/

static void smtp_transmit(int sock)
{

	FILE *sig;
	char buf[2048];
	EmailList *einfo, *last=email_list;
	int smtp=sock;

	for (einfo=email_list; einfo && einfo->sock != sock; last = einfo, einfo = einfo->next);

	if (!einfo) {
		if (debug)
			log("Couldn't find smtp email information for socket %d!", sock);
		return;
	}

	if (last==einfo)
		last=NULL;

	sig = fopen(SERVICES_SIG, "r");

	snprintf(buf, sizeof(buf), "EHLO %s\r\n", servername);
	send(smtp, buf, strlen(buf), 0);

       if (SMTP_LOGIN && SMTP_PASS) {
		snprintf(buf, sizeof(buf), "AUTH PLAIN\r\n");
		send(smtp, buf, strlen(buf), 0);
		snprintf(buf, sizeof(buf), "%s\r\n", SMTP_LOGIN);
		send(smtp, buf, strlen(buf), 0);
		snprintf(buf, sizeof(buf), "%s\r\n", SMTP_PASS);
		send(smtp, buf, strlen(buf), 0);
        }

	snprintf(buf, sizeof(buf), "MAIL FROM:<%s>\r\n", RETURN_EMAIL);
	send(smtp, buf, strlen(buf), 0);

	snprintf(buf, sizeof(buf), "RCPT TO:<%s>\r\n", einfo->email);
	send(smtp, buf, strlen(buf), 0);
	
	snprintf(buf, sizeof(buf), "DATA\r\n");
	send(smtp, buf, strlen(buf), 0);

	snprintf(buf, sizeof(buf), "From: %s <%s>\r\n", NETWORK_NAME, RETURN_EMAIL);
	send(smtp, buf, strlen(buf), 0);

	snprintf(buf, sizeof(buf), "To: <%s>\r\n", einfo->email);
	send(smtp, buf, strlen(buf), 0);

	snprintf(buf, sizeof(buf), "Subject: %s\r\n", einfo->subject);
	send(smtp, buf, strlen(buf), 0);


	if (einfo->append==0 && HIGH_NICK_SECURITY) {
		FILE *high = fopen(QUESTION_FILE, "r");
		if (high) {
			while (!feof(high)) {
				fgets(buf, 1024, high);
				send(smtp, buf, strlen(buf), 0);
			}
			fclose(high);
		}
	}
	snprintf(buf, sizeof(buf), "%s\n", einfo->info);
	send(smtp, buf, strlen(buf), 0);

	if (sig) {
		while (!feof(sig)) {
			fgets(buf, 1024, sig);
			send(smtp, buf, strlen(buf), 0);
		}
		fclose(sig);
	}

	snprintf(buf, sizeof(buf), "\r\n.\r\n");
	send(smtp, buf, strlen(buf), 0);

	snprintf(buf, sizeof(buf), "QUIT\r\n");
	send(smtp, buf, strlen(buf), 0);

	if (debug)
		log("Sent SMTP e-mail to %s on socket %d", einfo->email, einfo->sock);

	remove_emailinfo(einfo, last);
}

/************************************************************************/

void select_smtp()
{
	char sbuf[512];
	EmailList *einfo=email_list, *last=email_list, *next;
	int sdata=0;

	while (einfo) {
		next = einfo->next;
		if ((einfo->sock != -1) && FD_ISSET(einfo->sock, &smtp_read_fds)) {
			sdata = read(einfo->sock, &sbuf, 1);
			if (sdata == -1) {
				if (errno != EAGAIN) {
					Timeout *t;
					einfo->retries += 1;

					globops(NULL, "Couldn't establish connection to SMTP server. (%s)",
						strerror(errno));

					log("Couldn't establish connection to SMTP server. (%s)",
						strerror(errno));

					if (einfo->retries > SMTP_RETRIES) {
						log("Couldn't send email to %s after %d attempts.",
							einfo->email, SMTP_RETRIES);
						remove_emailinfo(einfo, last);
					} else {
						FD_CLR(einfo->sock, &smtp_read_fds);
						shutdown(einfo->sock, SHUT_RDWR);
						close(einfo->sock);
						einfo->sock = -1;
						t = add_timeout(SMTP_RESEND_TIME, resend_smtp, 0);
						t->data = einfo;
					}
				}

			} else if (sdata > 0) {
				smtp_transmit(einfo->sock);
}

		}
		last = einfo;
		einfo = next;
	}

}

/************************************************************************/
/************************************************************************/

