/*
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */

#include "services.h"
#include "log.h"


/*********************************************************************************************/
/*********************************************************************************************/

void do_help(const char *service, const char *source, char *topic, int level)
{
	FILE *helpfile;
	char tag[32], linebuf[256];
	char helppath[512];


	if (!topic)
		topic = "help";

	if (sstrcmp(topic, "sirv") == 0) {
		notice(s_HelpServ, source,
			"SirvNET Services are Copyright (c) 1998-2006 Trevor Klingbeil");
		notice(s_HelpServ, source, "Website: \2http://www.sirv.net\2");
		notice(s_HelpServ, source, "FTP: \2ftp://ftp.sirv.net\2");
		notice(s_HelpServ, source, "SirvNET Services written by Trevor Klingbeil <talinon@sirv.net>");
		return;

	} else if (sstrcmp(topic, "official") == 0) {
		ChanInfo *ci = official_list(NULL, 3);
		int shown=0, i=1;

		while (ci != NULL) {
			if (!shown) {
				notice(s_HelpServ, source,
					"\2%s OFFICIAL channel list\2:", NETWORK_NAME);
				shown=1;
			}
			notice(s_HelpServ, source,
				" %d)  %s", i++, ci->name);
				ci = official_list(ci, 3);
		}
		if (shown)
			notice(s_HelpServ, source, "\2End of OFFICIAL channel list\2.");
		else
			notice(s_HelpServ, source,
				"Currently no channels have been marked as official network channels.");
		return;
	}

	if (strlen(*&topic) > 32) {
		notice(service, source,
			"HELP ERROR: The topic you requested help on is way too long to be valid. "
			"For assistance, type - /%s HELP", s_HelpServ);
		return;
	}

	if (level && !is_oper(source)) {
		notice(service, source,
			"Sorry, no help available for '\2%s\2'.", topic);
		return;
	}

	snprintf(tag, sizeof(tag), ":%s:", topic);
	snprintf(helppath, sizeof(helppath), "%s/%s.help", HELPSERV_DIR,
		!sstrcmp(service, s_RootServ) ? s_OperServ : service);

	if ((helpfile = fopen(helppath, "rb")) == NULL) {
		log("Couldn't find Help file for %s (%s)", service, helppath);
		return;
	}

	while (!feof(helpfile)) {
		char *p;
		if (fgets(linebuf, 256, helpfile) == NULL)
			break;

		if (!(p = strtok(linebuf, "\r\n"))) /* blank line */
			continue;

		if (sstrcmp(p, tag)) /* not what we're looking for, try next line */
			continue;

		else  /* text tag matches help request. notice following lines until EOH */
			break;
	}

/* We shouldn't be at the end of file if valid help topic. Otherwise, let user know. */
	if (feof(helpfile)) {
		notice(service, source,
			"Sorry, no help available for '\2%s\2'.", topic);
		fclose(helpfile);
		return;
	}

/* Now we can process until EOH */

	while (!feof(helpfile)) {
		char *p;

		if (fgets(linebuf, 256, helpfile) == NULL)
			break;

		if (!(p = strtok(linebuf, "\r\n"))) {   /* blank line */
			notice(service, source, " ");
			continue;
		}

		if (sstrcmp(p, "EOH") == 0) {
			if (is_oper(source) && (sstrcmp(topic, "help") == 0) && sstrcmp(service, s_HelpServ)) {
				notice(service, source,
					"*** For a list of IRC Operator commands type: \2/%s OHELP\2",
						service);
			}
			if (level > 0 && level < 6)
				notice(service, source,
					"This command is limited to \2%s\2.",
				level==1 ? "IRC Operators" :
				level==2 ? "Services Operators" :
				level==3 ? "Services Administrators" :
				level==4 ? "Services Roots" :
				level==5 ? "Services Root Masters" : "");
			notice(service, source, "*** END OF HELP ***");
			fclose(helpfile);
			return;
		}

		notice(service, source, p);
	}
/* We should never end up here unless the help files have been modified inccorectly! */
	if (debug)
		log("HELP ERROR! Couldn't find EOH tag for topic '%s' in helpfile '%s'! "
			"Helpfiles tampored with or incorrectly modified!",
			topic, helppath);
	fclose(helpfile);
}


/*************************************************************************************************/
/*************************************************************************************************/

