/*
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */

#include "services.h"

/*********************************************************************/
/*********************************************************************/

nfreezes *freezelist = NULL;



void *scalloc(long elsize, long els)
{
    void *buf;

    if (!elsize || !els) {
        elsize = els = 1;
    }
    buf = (char *)calloc(elsize, els);

    return buf;
}


/*********************************************************************/


void *srealloc(void *ptr, size_t newsize)
{
	void *mbuf;

	if (!newsize)
		newsize=1;

	mbuf  = (char *)realloc(ptr, newsize);

	if (!mbuf)
		fatal_error("Out of memory!");

	return mbuf;
}


/*********************************************************************/


void *smalloc(size_t msize)
{
	void *mbuf;

	if (!msize)
		msize=1;

	mbuf = (char *)malloc(msize);

	if (!mbuf)
		fatal_error("Out of memory!");
	return mbuf;
}

/*********************************************************************/

char *sstrdup(const char *s)
{
	char *p = strdup(s);
	if (!p)
		fatal_error("Out of memory!");
	return p;
}


/*********************************************************************/

void freeze(const char *f, int freeze_t)
{
	nfreezes *nf;

	if (!freeze_t) {
		for (nf = freezelist; nf; nf = nf->next) {
			if (sstrcmp(nf->ent, f) == 0) {
				if (nf->prev)
					nf->prev->next = nf->next;
				else
					freezelist = nf->next;
				if (nf->next)
					nf->next->prev = nf->prev;
				free(nf->ent); free(nf);
				return;
			}
		}

	} else {
		nf = scalloc(sizeof(nfreezes), 1);
		nf->ent = sstrdup(f);
		nf->next = freezelist;
		if (freezelist)
			freezelist->prev = nf;
		freezelist = nf;
	}
}


/*********************************************************************/

void load_closures()
{
	int num_rows;
	CloseChans *cc;

	if ((num_rows=sql_select("name,expiry FROM cflags WHERE CLOSED > 0"))) {
		int i;
		for (i=0; i < num_rows; i++) {
			row = fetch(res);
			cc = scalloc(sizeof(CloseChans),1);
			cc->chan = sstrdup(row[0]);
			cc->expiry = atol(row[1]);
			cc->next = closelist;
			if (closelist)
				closelist->prev = cc;
			closelist = cc;
		}
	}
}

/*********************************************************************/

void load_freezes()
{
	int num_rows;
	nfreezes *nf;


#ifndef OLD_MYSQL
	if ((num_rows=sql_select("name FROM flags WHERE frozen > 0 UNION "
		"select name from cflags WHERE frozen > 0"))) {

#else
	if ((num_rows=sql_select("name FROM flags WHERE frozen > 0"))) {
#endif
		int i;
		for (i=0; i < num_rows; i++) {
			row = fetch(res);
			nf = scalloc(sizeof(nfreezes),1);
			nf->ent = sstrdup(row[0]);
			nf->next = freezelist;
			if (freezelist)
				freezelist->prev = nf;
			freezelist = nf;
		}
	}

#ifdef OLD_MYSQL
	if ((num_rows=sql_select("name FROM cflags WHERE frozen > 0"))) {
		int i;
		for (i=0; i < num_rows; i++) {
			row = fetch(res);
			nf = scalloc(sizeof(nfreezes),1);
			nf->ent = sstrdup(row[0]);
			nf->next = freezelist;
			if (freezelist)
				freezelist->prev = nf;
			freezelist = nf;
		}
	}
#endif

}


/*********************************************************************/

void load_akills()
{
	AKill *akill;
	char *ptr;
	int num_rows, host=0;

	if ((num_rows=sql_select("mask,id FROM akill"))) {
		int i;
		for (i=0; i < num_rows; i++, host=0) {
			row = fetch(res);
			akill = scalloc(sizeof(AKill),1);
			ptr = row[0];
			while (*ptr) {
				if (*ptr == '@')
					host=1;
				if (*ptr == '*') {
					if (!host)
						akill->flag |= AK_WILD_IDENT;
					else
						akill->flag |= AK_WILD_HOST;
				}
				*ptr++;
			}
			akill->mask = sstrdup(row[0]);
			akill->id = strtoul(row[1], NULL, 10);
			akill->next = akill_list;
			if (akill_list)
				akill_list->prev = akill;
			akill_list = akill;
		}
	}

}

/*********************************************************************/

void load_ignores()
{
	int num_rows;
	Ignore *ignore;

	if ((num_rows=sql_select("mask FROM silence"))) {
		int i;
		for (i=0; i < num_rows; i++) {
			row = fetch(res);
			ignore = scalloc(sizeof(Ignore),1);
			ignore->mask = sstrdup(row[0]);
			ignore->next = ignore_list;
			if (ignore_list)
				ignore_list->prev = ignore;
			ignore_list = ignore;
		}
	}

}

/*********************************************************************/
/*********************************************************************/

