/*
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */

#include "services.h"
#include "version.h"
#include "log.h"


static int message_routine(const char *nick);

unsigned long scommands=0;
unsigned long ucommands=0;

int pingburst=0;

/**************************************************************************************/
/**************************************************************************************/

static void m_version(char *source, int ac, char **av)
{
	User *u = finduser(source);

	if (FLOOD_PROT && u && do_update_flood(u, 1));

	if (source) {
		if (DISPLAY_VERSION)
			send_cmd(NULL, "351 %s %s %s : %s",
				source, version_number, servername, version_build);
		else
			send_cmd(NULL, "351 %s SirvNET 3 Services : %s",
				source, servername);
	}
	ucommands++;

}

/**************************************************************************************/

static void m_topic(char *source, int ac, char **av)
{
	if (ac != 4)
		return;

	do_topic(source, ac, av);

}

/**************************************************************************************/

/* Accept 'JOIN 0' from clients to part all channels. Anything else ignore since something
   is most likely broken on the network. */

static void m_join(char *source, int ac, char **av)
{
	User *u = finduser(source);

	if (ac != 1 || !u)
		return;

	if (!sstrcmp(av[0], "0"))   /* PART all channels */
		delete_uchans(u);

}

/**************************************************************************************/

static void m_mode(char *source, int ac, char **av)
{
	if ((*av[0] == '#') || (*av[0] == '&')) {
		if (ac < 2)
			return;
		c_mode(source, ac, av);

	} else {
		if (ac != 2)
			return;
		u_mode(source, ac, av);
	}

}

/**************************************************************************************/

static void m_motd(char *source, int ac, char **av)
{
	FILE *f;
	char buf[512];
	User *u = finduser(source);
	int num_rows, i, online=0;
	time_t uptime = time(NULL) - start_time;

	if (u && FLOOD_PROT)
		do_update_flood(u,2);

	send_cmd(servername, "375 %s :- %s (M)essage (O)f (T)he (D)ay",
		source, servername);
	snprintf(buf, sizeof(buf), "%s/%s", CONFDIR, MOTD_FILENAME);
	if ((f = fopen(buf, "r"))) {
		while (fgets(buf, sizeof(buf), f)) {
			buf[strlen(buf)-1] = 0;
			send_cmd(servername, "372 %s :- %s", source, buf);
		}
		fclose(f);
	}
	send_cmd(servername, "372 %s :-", source);

	send_cmd(servername, "372 %s :- This list of people can assist with lost "
		"passwords and registration matters:", source);

	send_cmd(servername, "372 %s :-", source);

	*buf=0;

	num_rows = sql_select("name FROM oper WHERE access >= 2");

	for (i=0; i < num_rows; i++) {
		row = fetch(res);
		if (finduser(row[0]) && is_recognized(row[0], row[0])) {
			strcat(buf, "\2");strcat(buf, row[0]);strcat(buf, "\2 ");
			online++;
		} else {
			strcat(buf, row[0]);strcat(buf, " ");
		}

		if (strlen(buf) > (512 - NICKMAX-4)) {
			send_cmd(servername, "372 %s :- %s", buf);
			*buf=0;
		}
	}

	if (*buf) {
		send_cmd(servername, "372 %s :- %s", source, buf);
		send_cmd(servername, "372 %s :-", source);
		send_cmd(servername, "372 %s :- Nicks in \2bold\2 represent currently on-line "
			"administrators.", source);
		send_cmd(servername, "372 %s :- \2%d\2 of %d nicks available", source, online, num_rows);

	} else
		send_cmd(servername, "372 :- List currently not available.");

	send_cmd(servername, "372 %s :-", source);

	if (MOTD_UPTIME) {
		if (uptime > 86400)
			send_cmd(servername, "372 %s :- "
				"Services up \2%d\2 day%s, \2%02d:%02d\2", source,
				uptime/86400, (uptime/86400 == 1) ? "" : "s",
				(uptime/3600) % 24, (uptime/60) % 60);
		else if (uptime > 3600)
			send_cmd(servername, "372 %s :- "
				"Services up \2%d hour%s, %d minute%s\2", source,
				uptime/3600, uptime/3600==1 ? "" : "s",
				(uptime/60) % 60, (uptime/60)%60==1 ? "" : "s");
		else
			send_cmd(servername, "372 %s :- "
				"Services up \2%d minute%s, %d second%s\2", source,
				uptime/60, uptime/60==1 ? "" : "s",
				uptime%60, uptime%60==1 ? "" : "s");
			
	}

	send_cmd(servername, "372 %s :-", source);
	send_cmd(servername, "372 %s :- SirvNET Services are copyright (c) "
		"1998-2006 Trevor Klingbeil.", source);

}

/**************************************************************************************/

static void m_ns(char *source, int ac, char **av)
{
	if (!source || !av[0])
		return;
	if (message_routine(source))
		nickserv(source, av[0]);
	ucommands++;
}

static void m_cs(char *source, int ac, char **av)
{
	if (!source || !av[0])
		return;
	chanserv(source, av[0]);
	ucommands++;
}

static void m_ms(char *source, int ac, char **av)
{
	if (!source || !av[0])
		return;
	memoserv(source, av[0]);
	ucommands++;
}

static void m_os(char *source, int ac, char **av)
{
	if (!source || !av[0])
		return;
	else if (!is_oper(source))
		notice(s_OperServ, source,
			"Access Denied: This service is only available to IRC Operators.");
	else if (message_routine(source))
		operserv(source, av[0]);
	ucommands++;
}

static void m_rs(char *source, int ac, char **av)
{
	if (!source || !av[0])
		return;
	else if (!is_oper(source))
		notice(s_RootServ, source,
			"Access Denied: This service is only available to IRC Operators.");
	else if (message_routine(source))
		rootserv(source, av[0]);
	ucommands++;
}

static void m_hs(char *source, int ac, char **av)
{
	if (!source || !av[0])
		return;
	do_help(s_HelpServ, source, av[0], 0);

	ucommands++;
}

static void m_ss(char *source, int ac, char **av)
{
	if (!source || !av[0])
		return;
	statserv(source, av[0]);
	ucommands++;
}


/**************************************************************************************/

static void m_privmsg(char *source, int ac, char **av)
{
	char *nick;

	if (ac != 2 || !source || !av[0])
		return;
	else if (!message_routine(source))
		return;

	if (strchr(av[0], '@')) {
		nick = strtok(av[0], "@");
		av[0] = nick;
	} else 	if (DISABLE_PRIVMSG) {
		notice(av[0], source,
			"\"/msg %s\" has been disabled by the network. Use /%s "
			"or /msg %s@%s instead.",
			av[0], av[0], av[0], servername);
		return;
	}

	if (sstrcmp(av[0], s_NickServ) == 0)
			nickserv(source, av[1]);
	else if (sstrcmp(av[0], s_ChanServ) == 0)
			chanserv(source, av[1]);
	else if (sstrcmp(av[0], s_MemoServ) == 0)
			memoserv(source, av[1]);
	else if (sstrcmp(av[0], s_HelpServ) == 0)
			do_help(s_HelpServ, source, av[1], 0);

	else if (sstrcmp(av[0], s_RootServ) == 0) {
		if (is_oper(source))
			rootserv(source, av[1]);
		else
			notice(s_RootServ, source,
			"Access Denied: This service is only available to IRC Operators.");

	} else if (sstrcmp(av[0], s_OperServ) == 0) {
		if (is_oper(source))
			operserv(source, av[1]);
		else
			notice(s_OperServ, source,
			"Access Denied: This service is only available to IRC Operators.");

	} else if (sstrcmp(av[0], s_StatServ) == 0)
			statserv(source, av[1]);
	ucommands++;
}

/**************************************************************************************/

static void m_part(char *source, int ac, char **av)
{
	if (!ac || ac > 2)
		return;
	do_part(source, ac, av);

}

/**************************************************************************************/


/* Bahamut uses a burst negotiation handshake using pings. On the third ping, send additional info. */


static void m_ping(char *source, int ac, char **av)
{
	if (ac < 1)
		return;

#ifdef BAHAMUT
	if (!uburst)
		++pingburst;

	if (!uburst && pingburst==3) {

		if (debug)
			log("Received user burst");

		uburst=1;pingburst=0;

		send_akills();
		send_sqlines();
		send_sglines();
	}

#endif

	send_cmd(servername, "PONG %s %s", ac>1 ? av[1] : remote_servername, av[0]);
}

/**************************************************************************************/

static void m_server(char *source, int ac, char **av)
{
	if (atoi(av[1]) == 1) {
		send_svinfo();
		add_server(av[0], NULL);
	} else {
		check_jupe(av[0]);
		add_server(source, av[0]);
	}

#ifdef SKELETON
	if (sstrcmp(av[0], RSERVICES_NAME) == 0)
		join_main();
#endif

}

/**************************************************************************************/

static void m_sqline(char *source, int ac, char **av)
{
	if (!sql_select("name FROM sqline WHERE name = '%s'", av[0])) {
		globops(NULL, "%s introduced SQLINE '%s'. Adding to database.",
			source, av[0]);
		send_sql("INSERT INTO sqline VALUES ('%s','%s','%s')",
			av[0], av[1], source);
		send_cmd(NULL, "SQLINE %s :%s", av[0], av[1]);
	}
}

/**************************************************************************************/

static void m_squit(char *source, int ac, char **av)
{
	squit_server(av[0]);
	if (sstrcmp(av[0], statsname) == 0) {
		send_cmd(NULL, "SERVER %s 2 :Statistical Database", statsname);
		globops(NULL, "%s tried to /SQUIT %s!", source, av[0]);
	}
}

/**************************************************************************************/

static void m_availability(char *source, int ac, char **av)
{
	User *u;
	int ulist=0;

	if (*av[1] != 0)
		fatal_error("Host network has an active %s client! This suggests "
		"either another set of services are running, or the host network "
		"is poorly configured. Please correct before running SirvNET Services "
		"again.", av[1]);
	load_pseudo_clients(NULL);


	if (debug)
		log("Processing user burst");
	for (ulist=0; ulist < 256; ulist++) {
		for (u = userlist[ulist]; u; u = u->next) {
			if (!u->timestamp) { /* otherwise user had already been thru this */
				newuser_routines(u);
				validate_user(u,1);
				check_memos(u,0);
			}
		}
	}
	if (debug) {
		log("Completed user burst");
		log("Processing channel burst...");
	}


	if (chanlist)
		process_channel_burst();
	else
		cburst=1;


	if (debug)
		log("Completed channel burst");


#ifdef SKELETON
	check_main();
#endif


#ifdef UNREAL  /* For bahamut we send these during a ping handshake. */
	send_sqlines();
	send_sglines();
	uburst=1;
	send_cmd(NULL, "NETINFO 0 %ld 0", time(NULL));
#endif
}


/**************************************************************************************/

static void show_uptime(const char *source) {

	time_t uptime = time(NULL) - start_time;

               if (uptime > 86400)
                        send_cmd(servername, "372 %s :- "
                                "Services up \2%d\2 day%s, \2%02d:%02d\2", source,
                                uptime/86400, (uptime/86400 == 1) ? "" : "s",
                                (uptime/3600) % 24, (uptime/60) % 60);
                else if (uptime > 3600)
                        send_cmd(servername, "372 %s :- "
                                "Services up \2%d hour%s, %d minute%s\2", source,
                                uptime/3600, uptime/3600==1 ? "" : "s",
                                (uptime/60) % 60, (uptime/60)%60==1 ? "" : "s");
                else
                        send_cmd(servername, "372 %s :- "
                                "Services up \2%d minute%s, %d second%s\2", source,
                                uptime/60, uptime/60==1 ? "" : "s",
                                uptime%60, uptime%60==1 ? "" : "s");
}


static void m_stats(char *source, int ac, char **av)
{
	if (ac != 2)
		return;

	switch(*av[0]) {
		case 'u':
		case 'U':
			if (MOTD_UPTIME)
				show_uptime(source);
			break;
		case 't':
		case 'T':
                        send_cmd(servername, "372 %s :- S: %d C: %d",
				source, scommands, ucommands);
			break;
			
	}


}

/**************************************************************************************/

#ifdef SOLID

static void do_svhost(char *source, int ac, char **av)
{
	User *u;


	if ((ac != 2) || !(u = finduser(av[0])))
		return;

	if (u->maskhost)
		free(u->maskhost);
	u->maskhost = sstrdup(av[1]);

}

#endif

/**************************************************************************************/


Message messages[] = {
	{ "436",	do_quit },
	{ "303",	m_availability },
	{ "AKILL",	NULL },
	{ "AWAY",	NULL },
	{ "CHATOPS",	NULL },
	{ "CS",		m_cs },
	{ "GLOBOPS",	NULL },
	{ "GNOTICE",	NULL },
	{ "HELP",	NULL },
	{ "HS",		m_hs },
	{ "JOIN",	m_join },
	{ "KICK",	do_kick },
	{ "KILL",	do_kill },
	{ "LUSERSLOCK", NULL },
	{ "MODE",	m_mode },
	{ "MOTD",	m_motd },
	{ "MS",		m_ms },
	{ "NICK",	do_nick },
	{ "NICKCOLL",	do_quit },
	{ "NOTICE",	NULL },
	{ "NS",		m_ns },
	{ "OS",		m_os },
	{ "PART",	m_part },
	{ "PASS",	NULL },
	{ "PING",	m_ping },
	{ "PRIVMSG",    m_privmsg },
	{ "QUIT",	do_quit },
	{ "RAKILL",	NULL },
	{ "RS",		m_rs },
	{ "SERVER",     m_server },
	{ "SJOIN",	m_sjoin },
	{ "SQUIT",	m_squit },
	{ "SQLINE",	m_sqline },
	{ "SS",		m_ss },
	{ "STATS",	m_stats },
#ifdef SOLID
	{ "SVHOST",	do_svhost },
#endif
	{ "SVSMODE",	NULL },
	{ "TOPIC",	m_topic },
	{ "VERSION",	m_version },
	{ "WALLOPS",	NULL },
	{ NULL }
};


Message *find_message(const char *name)
{
    Message *msg;

    for (msg = messages; msg->name; msg++) {
        if (sstrcmp(name, msg->name) == 0) {
            scommands++;
            return msg;
	}
    }
    return NULL;
}

/**************************************************************************************/

static int message_routine(const char *nick)
{
	User *user;

	if (!(user = finduser(nick)))
		return 0;

	else if (check_ignore(user->username, user->host) && !is_oper(nick))
		return 0;

	else if (FLOOD_PROT) {
		if (do_update_flood(user, 0) == 1)
			return 0;
	}
	return 1;

}


/**************************************************************************************/


void send_logonnews(User *u)
{
	FILE *f;
	char buf[512];


	snprintf(buf, sizeof(buf), "%s/%s", CONFDIR, LNEWS_FILENAME);

	if ((f = fopen(buf, "r"))) {
		send_cmd(servername, "375 %s :- %s Logon News",
			u->nick, NETWORK_NAME);
		while (fgets(buf, sizeof(buf), f)) {
			buf[strlen(buf)-1] = 0;
			send_cmd(servername, "372 %s :- %s", u->nick, buf);
		}
		fclose(f);
		send_cmd(servername, "372 %s :-", u->nick);
	}
}

/**************************************************************************************/
/**************************************************************************************/

