/*
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */


#include "services.h"
#include "log.h"

/*******************************************************************************************/
/*******************************************************************************************/

static char *get_chankey(const char *chan) {

	ChanInfo *ci = findchaninfo(chan);

	if (!ci)
		return NULL;

	if (sql_select("chankey FROM chan WHERE id = %d", ci->id)) {
		row = fetch(res);
		return row[0];
	} else
		return NULL;
}

/*******************************************************************************************/

static char *get_chanlimit(const char *chan) {

	ChanInfo *ci = findchaninfo(chan);

	if (!ci)
		return NULL;

	if (sql_select("chanlimit FROM chan WHERE id = %d", ci->id)) {
		row = fetch(res);
		return row[0];
	} else
		return NULL;
}

/*******************************************************************************************/

long def_mlocks(int setting)
{
	long modes=0;
	int set=0;

	if (strlen(DEF_MLOCKS) > 0) {
		char *ptr = DEF_MLOCKS;
		while (*ptr) {
			if (*ptr == '+') {
				if (setting)
					set=1;
				else
					set=0;
			} else if (*ptr == '-') {
				if (setting)
					set=0;
				else
					set=1;
			}

			if (!set) {
				*ptr++;
				continue;
			}

			switch(*ptr++) {
				case 'c':
					modes |= CMODE_C;
					break;
				case 'i':
					modes |= CMODE_I;
					break;
				case 'm':
					modes |= CMODE_M;
					break;
				case 'n':
					modes |= CMODE_N;
					break;
				case 'p':
					modes |= CMODE_P;
					break;
				case 's':
					modes |= CMODE_S;
					break;
				case 't':
					modes |= CMODE_T;
					break;
				case 'M':
					modes |= CMODE_m;
					break;
				case 'R':
					modes |= CMODE_r;
					break;
				default:
					break;
			}
		}
	}
	if (setting)
		modes |= CMODE_R;

	return modes;
}


char *get_cmodes(ChanInfo *ci)
{
	long modeson=0, modesoff=0;
	char modebuf[32], *end=modebuf, *p;


	if (!(sql_select("modeson,modesoff FROM chan WHERE id = %d", ci->id))) {
		return "+r";
	} else {
		row = fetch(res);
		modeson = atol(row[0]);
		modesoff = atol(row[1]);

		*end++ = '+'; *end++ = 'r';
		if (modeson & CMODE_C)
			*end++ = 'c';
		if (modeson & CMODE_I)
			*end++ = 'i';
		if (modeson & CMODE_K)
			*end++ = 'k';
		if (modeson & CMODE_L)
			*end++ = 'l';
		if (modeson & CMODE_M)
			*end++ = 'm';
		if (modeson & CMODE_N)
			*end++ = 'n';
		if (modeson & CMODE_P)
			*end++ = 'p';
		if (modeson & CMODE_S)
			*end++ = 's';
		if (modeson & CMODE_T)
			*end++ = 't';
		if (modeson & CMODE_m)
			*end++ = 'M';
		if (modeson & CMODE_o)
			*end++ = 'O';
		if (modeson & CMODE_r)
			*end++ = 'R';

		if (modesoff) {
			*end++ = '-';
			if (modesoff & CMODE_C)
				*end++ = 'c';
			if (modesoff & CMODE_I)
				*end++ = 'i';
			if (modesoff & CMODE_K)
				*end++ = 'k';
			if (modesoff & CMODE_L)
				*end++ = 'l';
			if (modesoff & CMODE_M)
				*end++ = 'm';
			if (modesoff & CMODE_N)
				*end++ = 'n';
			if (modesoff & CMODE_P)
				*end++ = 'p';
			if (modesoff & CMODE_S)
				*end++ = 's';
			if (modesoff & CMODE_T)
				*end++ = 't';
			if (modesoff & CMODE_m)
				*end++ = 'M';
			if (modesoff & CMODE_o)
				*end++ = 'O';
			if (modesoff & CMODE_r)
				*end++ = 'R';
		}
	}

	*end=0;
	return p=modebuf;
}

/*******************************************************************************************/


void check_modes(Channel *c, ChanInfo *ci)
{
	char *limit=0;
	int setkey=0, setlimit=0, freekey=0;

	if (!c)
		return;

	if (is_frozen(c->name))
		return;

	if (sql_select("modeson,modesoff FROM chan WHERE id = %d", ci->id)) {
		char modebuf[32], *end=modebuf;
		long modeson=0, modesoff=0;
		row = fetch(res);

		*modebuf=0;
		modeson = atol(row[0]);
		modesoff = atol(row[1]);

		*end++ = '+';

		if (modeson & CMODE_C) {
			if (!(c->modes & CMODE_C)) {
				c->modes |= CMODE_C;
				*end++ = 'c';
			}
		}
		if (modeson & CMODE_I) {
			if (!(c->modes & CMODE_I)) {
				c->modes |= CMODE_I;
				*end++ = 'i';
			}
		}
		if (modeson & CMODE_K) {
			char *key=get_chankey(c->name);
			if ((!(c->modes & CMODE_K)) ||
				(c->chankey && key && sstrcmp(c->chankey, key))) {
				c->modes |= CMODE_K;
				*end++ = 'k';
				setkey=1;
				if (key)
					c->chankey = sstrdup(key);
			}
		}
		if (modeson & CMODE_L) {
			limit=get_chanlimit(c->name);
			if (limit && ((!(c->modes & CMODE_L)) ||
				(c->chanlimit && c->chanlimit != atoi(limit)))) {
				c->modes |= CMODE_L;
				*end++ = 'l';
				setlimit=1;
				c->chanlimit = atoi(limit);

				if (limit)
					limit = sstrdup(limit);
			} else {
				modeson &= ~CMODE_L;
				send_sql("UPDATE chan SET modeson = %d WHERE id = %d",
					modeson, ci->id);
			}
		}
		if (modeson & CMODE_M) {
			if (!(c->modes & CMODE_M)) {
				c->modes |= CMODE_M;
				*end++ = 'm';
			}
		}
		if (modeson & CMODE_N) {
			if (!(c->modes & CMODE_N)) {
				c->modes |= CMODE_N;
				*end++ = 'n';
			}
		}
		if (modeson & CMODE_P) {
			if (!(c->modes & CMODE_P)) {
				c->modes |= CMODE_P;
				*end++ = 'p';
			}
		}
		if (modeson & CMODE_R) {
			if (!(c->modes & CMODE_R)) {
				c->modes |= CMODE_R;
				*end++ = 'r';
			}
		}

		if (modeson & CMODE_S) {
			if (!(c->modes & CMODE_S)) {
				c->modes |= CMODE_S;
				*end++ = 's';
			}
		}
		if (modeson & CMODE_T) {
			if (!(c->modes & CMODE_T)) {
				c->modes |= CMODE_T;
				*end++ = 't';
			}
		}
		if (modeson & CMODE_m) {
			if (!(c->modes & CMODE_m)) {
				c->modes |= CMODE_m;
				*end++ = 'M';
			}
		}
		if (modeson & CMODE_r) {
			if (!(c->modes & CMODE_r)) {
				c->modes |= CMODE_r;
				*end++ = 'R';
			}
		}
		if (modeson & CMODE_o) {
			if (!(c->modes & CMODE_o)) {
				c->modes |= CMODE_o;
				*end++ = 'O';
			}
		}

		if (end[-1] == '+')
			--end;
		*end++ = '-';

		if (modesoff & CMODE_C) {
			if (c->modes & CMODE_C) {
				c->modes &= ~CMODE_C;
				*end++ = 'c';
			}
		}

		if (modesoff & CMODE_I) {
			if (c->modes & CMODE_I) {
				c->modes &= ~CMODE_I;
				*end++ = 'i';
			}
		}

		if (modesoff & CMODE_K) {
			if (c->modes & CMODE_K) {
				c->modes &= ~CMODE_K;
				*end++ = 'k';
				setkey=1; freekey=1;
			}
		}

		if (modesoff & CMODE_L) {
			if (c->modes & CMODE_L) {
				c->modes &= ~CMODE_L;
				*end++ = 'l';
				c->chanlimit = 0;
			}
		}

		if (modesoff & CMODE_M) {
			if (c->modes & CMODE_M) {
				c->modes &= ~CMODE_M;
				*end++ = 'm';
			}
		}
		if (modesoff & CMODE_N) {
			if (c->modes & CMODE_N) {
				c->modes &= ~CMODE_N;
				*end++ = 'n';
			}
		}
		if (modesoff & CMODE_P) {
			if (c->modes & CMODE_P) {
				c->modes &= ~CMODE_P;
				*end++ = 'p';
			}
		}
		if (modesoff & CMODE_S) {
			if (c->modes & CMODE_S) {
				c->modes &= ~CMODE_S;
				*end++ = 's';
			}
		}
		if (modesoff & CMODE_T) {
			if (c->modes & CMODE_T) {
				c->modes &= ~CMODE_T;
				*end++ = 't';
			}
		}
		if (modesoff & CMODE_m) {
			if (c->modes & CMODE_m) {
				c->modes &= ~CMODE_m;
				*end++ = 'M';
			}
		}
		if (modesoff & CMODE_o) {
			if (c->modes & CMODE_o) {
				c->modes &= ~CMODE_o;
				*end++ = 'O';
			}
		}
		if (modesoff & CMODE_r) {
			if (c->modes & CMODE_r) {
				c->modes &= ~CMODE_r;
				*end++ = 'R';
			}
		}

		if (end[-1] == '-')
			--end;

		if (end == modebuf)
			return;

		*end=0;

		send_cmd(s_ChanServ, "MODE %s %s %s %s",
			c->name, modebuf, setkey ? c->chankey : "", setlimit ? limit : "");

	}

	if (freekey) {
		if (c->chankey)
			free(c->chankey);
		c->chankey = NULL;
	}

}


void u_mode(const char *source, int ac, char **av)
{
	User *user;
	char *modes;
	int add=1;

	if (sstrcmp(source, av[0])) {
		if (debug)
			log("user mode error: MODE %s %s from different nick %s",
				av[0], av[1], source);
		return;
	}

	user = finduser(source);
	if (!user) {
		if (debug)
			log("user mode error: MODE %s for nonexistent nick %s: %s",
				av[1], source, merge_args(ac, av));
	}
	if (debug)
		log("Changing mode for %s to %s", source, av[1]);

	modes = av[1];

	while (*modes) {
		switch(*modes++) {
			case '+': add = 1; break;
			case '-': add = 0; break;
			case 'o':
				if (add) {
					user->mode |= UMODE_O;
					opcnt++;
				} else {
					user->mode &= ~UMODE_O;
					opcnt--;
				}
				break;
			case 'r': add ? (user->mode |= UMODE_R) : (user->mode &= ~UMODE_R); break;

		}
	}


}

/*******************************************************************************************/

void c_mode(const char *source, int ac, char **av)
{
	ChanInfo *ci = findchaninfo(av[0]);
	Channel *c;
	char *chan = av[0];
#ifdef BAHAMUT
	char *modes = av[2];
	int add=0, acn=3;
#else /* UNREAL */
	char *modes = av[1];
	int add=0, acn=2;
#endif
	char *aptr;              /* modes will be length of 1 if new channel. Modes will be "+" */
	int checkmodes = (strlen(modes)==1) || (strspn(modes, "-+ovhbe") < strlen(modes));

	if (!(c = findchan(chan))) {
		if (ci && ci->flags & CS_SUSPEND) {
			send_cmd(s_ChanServ, "MODE %s +lb 1 *!*@*", chan);
			return;
		}
		if (debug)
			log("MODE: Nonexistent channel %s", chan);
		return;
	}

	while (*modes) {
		switch(*modes++) {
			case '+': add=1; break;
			case '-': add=0; break;


			case 'v':
				aptr = av[acn++];
				if (add)
					chan_addop(chan, aptr, 1);
				else
					chan_delop(chan, aptr, 1);
				break;

			case 'o':
				aptr = av[acn++];
				if (add) {
					if (!atol(av[1]) || check_valid_op(finduser(aptr), chan))
						chan_addop(chan, aptr, 0);
				} else {                                             /* samode = !atol(av[1] */
					if (!sstrcmp(source, aptr) || !atol(av[1]))
						chan_delop(chan, aptr, 0);

					else if (!check_op_protect(finduser(aptr), chan))
						chan_delop(chan, aptr, 0);

				}
				break;

			case 'l':
				aptr = av[acn++];
				if (add) {
					c->chanlimit = atoi(aptr);
					c->modes |= CMODE_L;
				} else {
					c->chanlimit = 0;
					c->modes &= ~CMODE_L;
				}


				break;


			case 'k':
				aptr = av[acn++];
				if (add) {
					if (c->chankey)
						free(c->chankey);
					c->chankey = sstrdup(aptr);
					c->modes |= CMODE_K;
				} else {
					if (c->chankey)
						free(c->chankey);
					c->chankey = NULL;
					c->modes &= ~CMODE_K;
				}
				break;

			case 'b':
				aptr = av[acn++];
				if (add)
					chan_addban(chan, aptr, 0);
				else
					chan_delban(chan, aptr);
				break;

#ifdef UNREAL
			case 'h':
				aptr = av[acn++];
				if (add)
					chan_addop(chan, aptr, 2);
				else
					chan_delop(chan, aptr, 2);
				break;
#endif

			case 'c':
				if (add)
					c->modes |= CMODE_C;
				else
					c->modes &= ~CMODE_C;
				break;


			case 'i':
				if (add)
					c->modes |= CMODE_I;
				else
					c->modes &= ~CMODE_I;
				break;

			case 'm':
				if (add)
					c->modes |= CMODE_M;
				else
					c->modes &= ~CMODE_M;
				break;

			case 'n':
				if (add)
					c->modes |= CMODE_N;
				else
					c->modes &= ~CMODE_N;
				break;


			case 'p':
				if (add)
					c->modes |= CMODE_P;
				else
					c->modes &= ~CMODE_P;
				break;


			case 'r':
				if (add)
					c->modes |= CMODE_R;
				else
					c->modes &= ~CMODE_R;
				break;

			case 'M':
				if (add)
					c->modes |= CMODE_m;
				else
					c->modes &= ~CMODE_m;
				break;

			case 'R':
				if (add)
					c->modes |= CMODE_r;
				else
					c->modes &= ~CMODE_r;
				break;

			case 's':
				if (add)
					c->modes |= CMODE_S;
				else
					c->modes &= ~CMODE_S;
				break;

			case 't':
				if (add)
					c->modes |= CMODE_T;
				else
					c->modes &= ~CMODE_T;
				break;

			case 'O':
				if (add)
					c->modes |= CMODE_o;
				else
					c->modes &= ~CMODE_o;
				break;

			case 'e':
			case 'I':
				acn++;
				break;

		}
	}
	if (cburst && ci && checkmodes)
		check_modes(c, ci);
}


/*******************************************************************************************/
/*******************************************************************************************/

