/*
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */

#include "services.h"
#include "log.h"

#include <dirent.h>

unsigned int last_news_id = 0;

/**********************************************************************************/
/**********************************************************************************/

static char *news_date(char *n_time)
{
	time_t t;
	struct tm tm;
	char date[256], *ptr;


	if (!n_time)
		return 0;

	t = atol(n_time);
	tm = *localtime(&t);

	strftime(date, sizeof(date)-1, "%b %d %Y", &tm);

	return ptr=date;

}

/**********************************************************************************/


void load_news()
{
	struct dirent *ent, *eptr;
	DIR *dir;
	FILE *f;
	unsigned int news_id=0, num_rows=0, i;
	char tbuf[512];


	if (chdir("./news") < 0) {
		system("mkdir ./news");
		if (chdir("./news") < 0) {
			log("Couldn't create news directory! (%s/news)", DATDEST);
			return;
		}
	}

	if (chdir("./data") < 0)
		system("mkdir ./data");
	else
		chdir("..");

	if (!(dir = opendir(".")))
		fatal_error("Couldn't open news directory! (%s/news)", DATDEST);


	if (!sql_select("news FROM sirv")) {
		log("Couldn't retrieve news id from sirv database!");
		chdir(DATDEST);
		return;
	}

	row = fetch(res);
	news_id = atoi(row[0]);

	while ((ent = readdir(dir)) != NULL) {
		if (ent->d_type != DT_DIR) {
			news_id++;
			snprintf(tbuf, sizeof(tbuf), "mv -f %s ./data/%d.news", ent->d_name, news_id);
			system(tbuf);
			send_sql("INSERT INTO news VALUES (%d,%d)", news_id, time(NULL));
		}
	}
	closedir(dir); 

	chdir("./data");
	if (!(dir = opendir(".")))
		fatal_error("Couldn't open news data directory! (%s/news/data)", DATDEST);

	while ((ent = readdir(dir)) != NULL) {
		if (ent->d_type != DT_DIR) {

			if (!strchr(ent->d_name, '.'))
				continue;
			eptr = ent;
			snprintf(tbuf, sizeof(tbuf), "rm -f %s", eptr->d_name);
			if (!sql_select("id FROM news WHERE id = %s", strtok(ent->d_name, ".")))
				system(tbuf);
		}
	}
	closedir(dir); chdir(DATDEST);


	if (!sql_select("* FROM news WHERE id = %d", news_id))
		last_news_id=0;  /* latest news has been deleted. Don't inform of any new network news. */
	else {
		send_sql("UPDATE sirv SET news = %d", news_id);
		last_news_id = news_id;
	}


	if ((num_rows = sql_select("id FROM news"))) {
		for (i=0; i < num_rows; i++) {
			row = fetch(res);
			snprintf(tbuf, sizeof(tbuf), "%s/news/data/%s.news",
				DATDEST, row[0]);
			if (!(f = fopen(tbuf, "r"))) {
				send_sql("DELETE FROM news WHERE id=%s", row[0]);
			} else
				fclose(f);
		}
	}

}

/**********************************************************************************/

E void do_news(const char *source)
{
	const char *param = strtok(NULL, " ");
	int num_rows = sql_select("* FROM news ORDER BY time");


	if (param && sstrcmp(param, "LIST") == 0) {
		if (!num_rows)
			notice(s_MemoServ, source, "There is currently no news available to read.");

		else {
			int i;
			notice(s_MemoServ, source, "\37%s news listings\37:", NETWORK_NAME);

			for (i=0; i < num_rows; i++) {
				row = fetch(res);
				notice(s_MemoServ, source,
					" \2%d\2) %s   -- news update #%s", i+1, news_date(row[1]), row[0]);
			}
			notice(s_MemoServ, source,
				"End of news listing.");

		}
	} else if (param && (!strspn(param, "0123456789") || (atoi(param) > num_rows) || (atoi(param) < 1))) {
			notice(s_MemoServ, source, "Invalid news index number.");

	} else if (!num_rows) {
			notice(s_MemoServ, source, "There is currently no news available to read.");

	} else {
		FILE *f;
		char fbuf[strlen(DATDEST) + 32];
		int index_n=0;

		if (!param)
			index_n = num_rows;
		else
			index_n = atoi(param);

		mysql_data_seek(res, index_n-1);
		row = fetch(res);

		snprintf(fbuf, sizeof(fbuf), "%s/news/data/%s.news", DATDEST, row[0]);
		f = fopen(fbuf, "r");

		if (!f) {
			notice(s_MemoServ, source,
				"Could not retrieve news for index #%d. ", index_n);
			load_news();
		} else {
			char linebuf[256], *p;

			notice(s_MemoServ, source,
				"News for \2%s\2:", news_date(row[1]));

			notice(s_MemoServ, source, " ");

			while (!feof(f)) {
				if (fgets(linebuf, 256, f) == NULL)
					break;

				if (!(p = strtok(linebuf, "\r\n"))) {
					notice(s_MemoServ, source, " ");
					continue;
				}
				notice(s_MemoServ, source, p);
			}
			fclose(f);

			notice(s_MemoServ, source,
				"End of news update %d. (Historical #%s)", index_n, row[0]);

			if ((unsigned int)atoi(row[0]) == last_news_id) {
				User *u = finduser(source);
				struct u_idlist *nl;

				u->news = last_news_id;
				for (nl = u->nicks; nl; nl = nl->next)
					send_sql("UPDATE nick SET news=%d WHERE name = '%s'",
					last_news_id, nl->nick);
			}
		}

	}
}


/**********************************************************************************/
/**********************************************************************************/

