/*
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 * Parts of this file are originally based on EsperNet Services(c) by Andy Church.
 */



#include "services.h"
#include "log.h"

/******************************************************************/
/******************************************************************/


/* split_buf:  Split a buffer into arguments and store the arguments in an
 *             argument vector pointed to by argv (which will be malloc'd
 *             as necessary); return the argument count.  If colon_special
 *             is non-zero, then treat a parameter with a leading ':' as
 *             the last parameter of the line, per the IRC RFC.  Destroys
 *             the buffer by side effect.
 */

static int split_buf(char *buf, char ***argv, int colon_special)
{
    int argvsize = 8;
    int argc;
    char *s;

    *argv = smalloc(sizeof(char *) * argvsize);
    argc = 0;
    while (*buf) {
        if (argc == argvsize) {
            argvsize += 8;
            *argv = srealloc((char *)*argv, sizeof(char *) * argvsize);
        }
        if (*buf == ':') {
            (*argv)[argc++] = buf+1;
            buf = "";
        } else {
            s = strpbrk(buf, " ");
            if (s) {
                *s++ = 0;
                while (isspace(*s))
                   s++;
            } else {
                s = buf + strlen(buf);
            }
            (*argv)[argc++] = buf;
            buf = s;
        }
    }
    return argc;
}




/******************************************************************/

void process(int target)
{

	char buf[512];
	char source[64];
	char cmd[64];

	int ac;
	char **av;
	char *str;
	Message *msg;



	if (debug_traffic)
		log("(%s) Received: %s", target==servfd ? "services" : "stats", inbuf);

	strncpy(buf, inbuf, sizeof(buf));

	if (*buf == ':') {
		str = strpbrk(buf, " ");
		if (!str)
			return;
		*str = 0;
		while (isspace(*++str))
			;
		strncpy(source, buf+1, sizeof(source));
		strcpy(buf, str);
	} else
		*source = 0;
	if (!*buf)
		return;
	str = strpbrk(buf, " ");
	if (str) {
		*str = 0;
		while (isspace(*++str))
			;
	} else
		str = buf + strlen(buf);

	strncpy(cmd, buf, sizeof(cmd));

	ac = split_buf(str, &av, 1);

	if ((msg = find_message(cmd))) {
		if (msg->func)
			msg->func(source, ac, av);
	} else if (debug)
		log("Unknown message from server (%s)", inbuf);

	free(av);

}

/******************************************************************/

/* Updates flood levels of a user. If exceeds max, warn, if continues,
   kill user */

int do_update_flood(User *u, unsigned short idflood)
{
	time_t now = time(NULL);

	if (u->flood_time < now) {
		u->flood_time = now + F_RESET;
		u->floodlev = 0;

	} else if (u->flood_time > now) {
		u->flood_time = now + F_RESET;
		u->floodlev += 1;
	}
	if (idflood == 2)
		u->motd += 1;
	else if (idflood == 1)
		u->version += 1;

	if ((u->motd == 30) || (u->version == 30)
	|| (u->motd == 50) || (u->version == 50)) {
		globops(NULL, "WARNING: User %s!%s@%s has been found "
			"sending atleast %d %s requests to services",
			u->nick, u->username, u->host,
			u->motd == 30 ? u->motd : u->version,
			u->motd == 30 ? "MOTD" : "VERSION");
		notice(s_Global, u->nick,
			"You have been constantly sending %s "
			"requests to services. Please stop immediately.",
			u->motd == 30 ? "MOTD" : "VERSION");
		if (idflood == 2)         
			u->motd += 1;       /*      to prevent repeated notices */
		else if (idflood == 1)    
			u->version += 1;     
	}
	if ((u->floodlev == 5) && (u->floodlev2 == 3)) {
		u->floodlev2 = 4;
		if (is_oper(u->nick))
			globops(NULL, "Oper %s has been SEVERELY flooding services", u->nick);
		if (!is_oper(u->nick)) {
			kill_user(s_Global, u->nick, "Stop flooding services");
			return 1;
		}
	} else if ((u->floodlev == F_TLEV) && (u->floodlev2 == 2)) {
		u->floodlev2 = 3;
		u->floodlev = 1;
		notice(s_Global, u->nick, "You have been warned twice. STOP FLOODING SERVICES");
		if (!is_oper(u->nick))
			globops(NULL, "Services are being \2SEVERELY\2 flooded by %s!%s@%s",
				u->nick, u->username, u->host);

	} else if ((u->floodlev == F_TLEV) && (u->floodlev2 == 1)) {
		u->floodlev2 = 2;
		u->floodlev = 1;
		notice(s_Global, u->nick, "You are flooding services, please slow down");
		if (!is_oper(u->nick))
			globops(NULL, "Services are being flooded by %s!%s@%s",
				u->nick, u->username, u->host);

	} else if ((u->floodlev == 8) && (u->floodlev2 == 0))
		u->floodlev2 = 1;
	return 0;
}




/******************************************************************/
/******************************************************************/

