/*    NO QUIT CAPAB
 *
 * SirvNET Services are copyright (c) 1998-2006 Trevor Klingbeil.
 *    http://www.sirv.net  ::  ftp.sirv.net
 * This program is copyrighted software; see the file COPYING for details.
 */

#include "services.h"
#include "log.h"


ServerList *myhub = NULL;

/****************************************************************************************/
/****************************************************************************************/

/* If leaf is NULL then this will be the services' hub server */

void add_server(const char *hub, const char *leaf)
{

	ServerList *sl, *hubptr = NULL, *last = NULL, *slh;


	for (sl = myhub; sl; sl = sl->next) {
		if (leaf && sstrcmp(sl->name, leaf) == 0) {
			if (debug)
				log("SERVER: %s introduced already present server %s!", hub, leaf);
			return;
		}
		if (sstrcmp(sl->name, hub) == 0)
			hubptr = sl;
		last = sl;
	}
	if (myhub && !hubptr) {
		if (debug)
			log("SERVER: Non-existent hub server %s introduced leaf server %s!", hub, leaf);
		return;
	}

	sl = scalloc(sizeof(struct server_list), 1);
	if (!leaf) {
		sl->name = sstrdup(hub);
		sl->hub = NULL; sl->prev = NULL;
		myhub = sl;
	} else {
		sl->name = sstrdup(leaf);
		sl->hub = hubptr;
	}
	sl->next = NULL;
	sl->ndownlinks = 0;
	sl->downlink_size = 0;
	sl->nusers = 0;
	sl->users_size = 0;

	if (last) {
		last->next = sl;
		sl->prev = last;
	}

	/* propogate downlinks */

	if (hubptr) {
		for (slh = hubptr; slh; slh = slh->hub) {
			if (slh->ndownlinks >= slh->downlink_size) {
				if (!(slh->downlink_size)) {  /* newly allocate memory for downlinks */
					slh->downlink_size = 8;
					slh->downlinks = smalloc(sizeof(*slh->downlinks) * 8);

				} else {   /* reallocate more memory for downlinks */
					slh->downlink_size += 8;
					slh->downlinks = srealloc(slh->downlinks,
						sizeof(*slh->downlinks) * slh->downlink_size);
				}
			}
			slh->downlinks[slh->ndownlinks++] = sl;
		}
	}
}


/****************************************************************************************/

static void remove_downlinks(ServerList *server, ServerList *sserver)
{
	ServerList *sl;
	int i;

	for (sl = server->hub; sl; sl = sl->hub) {
		for (i=0; i < sl->ndownlinks; i++) {
			if (sl->downlinks[i] == sserver) {
				sl->ndownlinks--;
				if (i < sl->ndownlinks)
					memcpy(sl->downlinks+i, sl->downlinks+i+1,
						sizeof(*sl->downlinks) * (sl->ndownlinks-i));
				break;
			}
		}
	}

}

/****************************************************************************************/

static void delete_server(ServerList *sl)
{
	int i;

#ifdef SKELETON
	if (!sstrcmp(sl->name, RSERVICES_NAME))
		split_main();
#endif


	for (i=0; i < sl->nusers; i++)
		delete_user(sl->users[i]);

	if (sl->prev)
		sl->prev->next = sl->next;
	else
		myhub = NULL;
	if (sl->next)
		sl->next->prev = sl->prev;
	free(sl->name);
	free(sl->downlinks);
	free(sl->users);
	free(sl);

}

/****************************************************************************************/

void squit_server(const char *server)
{
	ServerList *sl;
	int i;

	for (sl = myhub; sl; sl = sl->next) {

		if (sstrcmp(sl->name, server) == 0) {
			for (i=0; i < sl->ndownlinks; i++)
				remove_downlinks(sl, sl->downlinks[i]);
			remove_downlinks(sl, sl);
			for (i=0; i < sl->ndownlinks; i++)
				delete_server(sl->downlinks[i]);
			delete_server(sl);
			break;
		}
	}
}

/****************************************************************************************/

/*
 * This is used for all other user exits except for server splits.
 * The pointer to pointer in the server's user list is removed for the exiting client.
 */

void user_del_server(User *u)
{
	ServerList *sl = u->slptr;
	int i;

	for (i=0; i < sl->nusers; i++) {
		if (u == sl->users[i]) {
			--sl->nusers;
			if (i < sl->nusers)
				memcpy(sl->users+i, sl->users+i+1,
					sizeof(*sl->users) * (sl->nusers-i));
			break;
		}
	}
}

/****************************************************************************************/

void user_add_server(User *u)
{
	ServerList *sl;

	for (sl = myhub; sstrcmp(sl->name, u->server); sl = sl->next) ;


	if (!sl) {
		if (debug)
			log("USER: Client '%s' from non-existent server %s!", u->nick, u->server);
		return;
	}

	if (sl->nusers >= sl->users_size) {
		if (!sl->users_size) {
			sl->users_size = 32;
			sl->users = smalloc(sizeof(*sl->users) * 32);
		} else {
			sl->users_size += 32;
			sl->users = srealloc(sl->users, sizeof(*sl->users) * sl->users_size);
		}
	}
	sl->users[sl->nusers++] = u;
	u->slptr = sl;
}

/****************************************************************************************/

#ifdef SKELETON


void split_main()
{
	globops(NULL, "Main services server %s not detected - Enabling back-up oper services.",
		RSERVICES_NAME);
	skeleton_mode = 1;

}

/****************************************************************************************/

void join_main()
{

	globops(NULL, "Main services server %s detected - back-up oper services going into dormant mode.",
		RSERVICES_NAME);
	skeleton_mode = 0;

}


/****************************************************************************************/

void check_main()
{
	ServerList *sl;


	for (sl = myhub; sl; sl = sl->next) {
		if (sstrcmp(sl->name, RSERVICES_NAME) == 0)
			return;
	}

	split_main();
}

#endif

/****************************************************************************************/
/****************************************************************************************/

